// SPDX-License-Identifier: (GPL-2.0-or-later OR BSD-2-Clause)
/*
 * Traceshark - a visualizer for visualizing ftrace and perf traces
 * Copyright (C) 2015, 2016  Viktor Rosendahl <viktor.rosendahl@gmail.com>
 *
 * This file is dual licensed: you can use it either under the terms of
 * the GPL, or the BSD license, at your option.
 *
 *  a) This program is free software; you can redistribute it and/or
 *     modify it under the terms of the GNU General Public License as
 *     published by the Free Software Foundation; either version 2 of the
 *     License, or (at your option) any later version.
 *
 *     This program is distributed in the hope that it will be useful,
 *     but WITHOUT ANY WARRANTY; without even the implied warranty of
 *     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *     GNU General Public License for more details.
 *
 *     You should have received a copy of the GNU General Public
 *     License along with this library; if not, write to the Free
 *     Software Foundation, Inc., 51 Franklin St, Fifth Floor, Boston,
 *     MA 02110-1301 USA
 *
 * Alternatively,
 *
 *  b) Redistribution and use in source and binary forms, with or
 *     without modification, are permitted provided that the following
 *     conditions are met:
 *
 *     1. Redistributions of source code must retain the above
 *        copyright notice, this list of conditions and the following
 *        disclaimer.
 *     2. Redistributions in binary form must reproduce the above
 *        copyright notice, this list of conditions and the following
 *        disclaimer in the documentation and/or other materials
 *        provided with the distribution.
 *
 *     THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND
 *     CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES,
 *     INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF
 *     MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 *     DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR
 *     CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 *     SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT
 *     NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
 *     LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 *     HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 *     CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR
 *     OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE,
 *     EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

#ifndef WORKTHREAD_H
#define WORKTHREAD_H

#include <QString>
#include "threads/tthread.h"
#include "misc/traceshark.h"

#define WORKTHREAD_DEFAULTNAME "WorkThread"

template <class ObjType>
class WorkThread : public TThread
{
public:
	WorkThread();
	WorkThread(const QString &name);
	WorkThread(ObjType *p, DEFINE_MEMBER_FN(void, ObjType, oF));
	WorkThread(const QString &name, ObjType *p,
		   DEFINE_MEMBER_FN(void, ObjType, oF));
	~WorkThread();
	void setObjFn(ObjType *p, DEFINE_MEMBER_FN(void, ObjType, oF));
protected:
	void run();
private:
	ObjType *workObject;
	DEFINE_MEMBER_FN(void, ObjType, objectFunc);
};

template <class ObjType>
WorkThread<ObjType>::WorkThread():
TThread(QString(WORKTHREAD_DEFAULTNAME)), workObject(nullptr),
	objectFunc(nullptr) {}

template <class ObjType>
WorkThread<ObjType>::WorkThread(const QString &name):
TThread(name), workObject(nullptr), objectFunc(nullptr) {}

template <class ObjType>
WorkThread<ObjType>::WorkThread(ObjType *p,
				DEFINE_MEMBER_FN(void, ObjType, oF)):
TThread(QString(WORKTHREAD_DEFAULTNAME)), workObject(p), objectFunc(oF) {}

template <class ObjType>
WorkThread<ObjType>::WorkThread(const QString &name, ObjType *p,
				DEFINE_MEMBER_FN(void, ObjType, oF)):
TThread(name), workObject(p), objectFunc(oF) {}


template <class ObjType>
void WorkThread<ObjType>::run()
{
	CALL_MEMBER_FN(workObject, objectFunc)();
}

template <class ObjType>
WorkThread<ObjType>::~WorkThread()
{
}

template <class ObjType>
void WorkThread<ObjType>::setObjFn(ObjType *p,
				   DEFINE_MEMBER_FN(void, ObjType, oF)) {
	workObject = p;
	objectFunc = oF;
}

#endif /* WORKTHREAD */
